@@ -1,16 +1,14 @@
-'''WebGeoCalc module.'''
-
-import pbr.version
-
-__version__ = pbr.version.VersionInfo('webgeocalc').version_string()
+"""WebGeoCalc module."""
 
 from .api import API, Api, ESA_API, JPL_API
 from .calculation import Calculation
 from .calculation_types import (AngularSeparation, AngularSize,
-                                FrameTransformation, IlluminationAngles,
-                                OsculatingElements, StateVector,
-                                SubObserverPoint, SubSolarPoint,
+                                FrameTransformation, GFCoordinateSearch,
+                                IlluminationAngles, OsculatingElements,
+                                StateVector, SubObserverPoint, SubSolarPoint,
                                 SurfaceInterceptPoint, TimeConversion)
+from .version import __version__
+
 
 __all__ = [
     'Api',
@@ -21,6 +19,7 @@ __all__ = [
     'AngularSize',
     'Calculation',
     'FrameTransformation',
+    'GFCoordinateSearch',
     'IlluminationAngles',
     'OsculatingElements',
     'StateVector',
